/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago4q.extractors.clearquest;

import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.extractors.AbstractExtractor;
import it.eng.spago4q.extractors.GenericItemSetterInterface;
import it.eng.spago4q.extractors.bo.GenericItemInterface;
import it.eng.spago4q.extractors.clearquest.CQWrapperSSH;
import it.eng.spago4q.extractors.xml.SaxElementHandler;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringEscapeUtils;
import org.apache.log4j.Logger;
import org.dom4j.DocumentException;
import org.dom4j.io.SAXReader;

public class ClearQuestExtractor extends AbstractExtractor implements
		GenericItemSetterInterface {

	private static final String XPATH = "/entities/entity";
	private static final String ATTRIBUTE = "ATTRIBUTE";
	
	private static final String ENTITY = "ENTITY";
	
	private String entryName;
	private static Logger logger = Logger.getLogger(ClearQuestExtractor.class);

	private List<GenericItemInterface> result = new ArrayList<GenericItemInterface>();
	private InputSourceBeanHandler inputSourceBeanHandler;

	protected void generateResult(String outputXml) throws EMFUserError {
		logger.debug("IN");
		SAXReader reader = new SAXReader();
		reader.addHandler(XPATH, new SaxElementHandler(this));
		try {
			reader.read(new ByteArrayInputStream(outputXml.getBytes()));
		} catch (DocumentException e) {
			logger.error(e.getLocalizedMessage());
			logger.debug(e.getStackTrace());
			throw new EMFUserError(EMFErrorSeverity.ERROR, e.getMessage());
		} finally {
			logger.debug("OUT");
		}
	}

	public void setGenericItem(final GenericItemInterface genericItem) {
		result.add(genericItem);
	}

	@Override
	protected List<GenericItemInterface> extract() throws EMFUserError {
		logger.debug("IN");

		String xmlRequest = inputSourceBeanHandler.getInputSourceBeanString();
		logger.debug("CQ Request XML: " + xmlRequest);
		
		try {
			CQWrapperSSH wrapper = new CQWrapperSSH();
			wrapper.setHost("161.27.204.80");

			// Definizione parametri di connessione (Opzionale)
//			wrapper.setHost("localhost");
//			wrapper.setPort(7070);
//			wrapper.setTimeout(2000);

			String xmlOut = wrapper.getQueryResult(xmlRequest);

			// TODO remove the System.out
//			System.out.println("PROBLEM:");
//			System.out.println(xmlOut);

			generateResult(xmlOut);

		} catch (IOException e) {
			logger.error(e.getLocalizedMessage());
			logger.debug(e.getStackTrace());
			throw new EMFUserError(EMFErrorSeverity.ERROR, "IO Error:"
					+ e.getStackTrace() + e.getLocalizedMessage());
		} catch (InterruptedException e) {
			logger.error(e.getLocalizedMessage());
			logger.debug(e.getStackTrace());
			throw new EMFUserError(EMFErrorSeverity.ERROR, "Interrupted Error:"
					+ e.getStackTrace());
		}
		logger.debug("OUT");
		return result;
	}

	@Override
	protected void setUp() {
		logger.debug("IN");
		entryName = readOperationParameterValue(ENTITY);
		Date lastOperationExtractionDate = null;
		try {
			lastOperationExtractionDate = getLastExtractionDate();
		} catch (EMFUserError e) {
			logger.debug(e.getLocalizedMessage());
			logger.debug(e.getStackTrace());
		}
		inputSourceBeanHandler = new InputSourceBeanHandler(entryName,
				lastOperationExtractionDate);
		inputSourceBeanHandler
				.setInputSourceBean(readOperationParameterValueList(ATTRIBUTE));
		logger.debug("OUT");
	}

	@Override
	protected void tearDown() {
	}

	@Override
	public String operationTest(Integer operationId) throws EMFUserError {
		logger.debug("IN");
		setOperationParameter(operationId);
		String toReturn = "The XML input is:<br>";
		toReturn += "<textarea rows=\"5\" cols=\"30\" readonly>";
		Date lastOperationExtractionDate = null;
		try {
			lastOperationExtractionDate = getLastExtractionDate();
		} catch (EMFUserError e) {
			logger.debug(e.getMessage());
			logger.debug(e.getStackTrace());
		}
		InputSourceBeanHandler tempImputSourceBean = new InputSourceBeanHandler(
				readOperationParameterValue(ENTITY), lastOperationExtractionDate);
		tempImputSourceBean
				.setInputSourceBean(readOperationParameterValueList(ATTRIBUTE));
		toReturn += StringEscapeUtils.escapeHtml(tempImputSourceBean
				.getInputSourceBeanString().replace('\n', ' '));
		toReturn += "</textarea>";
		logger.debug("TEST OPERATION RESULT: " + toReturn);
		logger.debug("OUT");
		return toReturn;
	}
}
